// ArrayMakerDlg.cpp : implementation file
//

#include "stdafx.h"
#include "ArrayMaker.h"
#include "ArrayMakerDlg.h"
#include "help.h"
#include <math.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CArrayMakerDlg dialog

CArrayMakerDlg::CArrayMakerDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CArrayMakerDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CArrayMakerDlg)
	m_nDiameter = 100;
	m_nHoleSize = 32;
	m_nXPos = 1000;
	m_nYPos = 1000;
	m_nQuantity = 6;
	m_nAngle = 360;
	m_nRadius = 0;
	m_nStartAngle = 0;
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CArrayMakerDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CArrayMakerDlg)
	DDX_Text(pDX, IDC_DIAMETER, m_nDiameter);
	DDV_MinMaxUInt(pDX, m_nDiameter, 1, 1000);
	DDX_Text(pDX, IDC_HOLE, m_nHoleSize);
	DDV_MinMaxUInt(pDX, m_nHoleSize, 0, 1000);
	DDX_Text(pDX, IDC_X_CENTRE, m_nXPos);
	DDV_MinMaxUInt(pDX, m_nXPos, 0, 32000);
	DDX_Text(pDX, IDC_Y_CENTRE, m_nYPos);
	DDV_MinMaxUInt(pDX, m_nYPos, 0, 32000);
	DDX_Text(pDX, IDC_NUMBER, m_nQuantity);
	DDV_MinMaxUInt(pDX, m_nQuantity, 0, 1000);
	DDX_Text(pDX, IDC_ANGLE, m_nAngle);
	DDV_MinMaxUInt(pDX, m_nAngle, 0, 360);
	DDX_Text(pDX, IDC_RADIUS, m_nRadius);
	DDV_MinMaxUInt(pDX, m_nRadius, 0, 16000);
	DDX_Text(pDX, IDC_STARTING, m_nStartAngle);
	DDV_MinMaxUInt(pDX, m_nStartAngle, 0, 360);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CArrayMakerDlg, CDialog)
	//{{AFX_MSG_MAP(CArrayMakerDlg)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_CREATE, OnCreate)
	ON_BN_CLICKED(IDC_BUTTON_HELP, OnButtonHelp)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CArrayMakerDlg message handlers

BOOL CArrayMakerDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	
	// TODO: Add extra initialization here
	m_pOutFile = new CStdioFile;
	CFileException exception;
	m_bFileOpen = m_pOutFile->Open("Output.PCB", CFile::modeCreate | CFile::modeWrite, &exception); 
	if(!m_bFileOpen)
	{
		MessageBox("Sorry - can't open file for writing. Don't know why - it just happens sometimes", "Oops!");
		return false;
	}
	CString str;
	str.LoadString(IDS_PCBFILE4);
	m_pOutFile->WriteString(str);
	m_pOutFile->WriteString("\n");
	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CArrayMakerDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CArrayMakerDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}

void CArrayMakerDlg::OnOK() 
{
	if(m_bFileOpen)
	{
		CString str;
		str.LoadString(IDS_ENDPCB);
		m_pOutFile->WriteString(str);
		m_pOutFile->WriteString("\n");
		m_pOutFile->Close();
	}
	
	CDialog::OnOK();
}

void CArrayMakerDlg::OnCancel() 
{
	if(m_bFileOpen)
		m_pOutFile->Close();
	CDialog::OnCancel();
}

void CArrayMakerDlg::OnCreate() 
{
	char line[100];
	static int padnum = 0;
	
	int x_offset;
	int y_offset;

	if(!m_bFileOpen)
		return;

	UpdateData(true);
	UINT count;
	double angle_inc, angle;
	angle_inc = (double) m_nAngle / (double) m_nQuantity;

	for(count = padnum; count < (padnum + m_nQuantity); count++)
	{
		angle = (double) m_nStartAngle + (double) (count - padnum) * angle_inc;
		angle = angle * 3.14159265 / 180;
		x_offset = (int) ((double) m_nRadius * cos(angle));
		y_offset = (int) ((double) m_nRadius * sin(angle));	
		wsprintf(line, "%d %d %d %d 1 %d 1 13\n", m_nXPos + x_offset, m_nYPos + y_offset, m_nDiameter, m_nDiameter,	m_nHoleSize);
		m_pOutFile->WriteString("FP\n");
		m_pOutFile->WriteString(line);
		wsprintf(line, "%d\n", count + 1);
		m_pOutFile->WriteString(line);
		m_pOutFile->WriteString("FA\n");
		wsprintf(line, "%d %d %d 15 10 7\n", m_nXPos + x_offset, m_nYPos + y_offset, m_nDiameter / 2 + 10);
		m_pOutFile->WriteString(line);
	}
	padnum += m_nQuantity;								
}

void CArrayMakerDlg::OnButtonHelp() 
{
	CHelp dlg;
	dlg.DoModal();
}
